<?php

// MinimalWeblog 1.0.2
// Copyright 2006 Jan Pieter Kunst <jpkunst at xs4all dot nl>
// 
// Based on Personal Weblog 1.0.0
// http://www.kyne.com.au/~mark/software/weblog.php
// Copyright 2001,2002  Mark Pulford <mark@kyne.com.au>
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version. 
//  
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details. 
//  
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

/**
 * @package MinimalWeblog
 * @version 1.0.2
 */

/**
 * Class to make a connection to a MySQL database with the mysqli extension
 *
 * @package MinimalWeblog
 */
class DB_MySQLi {

	/**
	 * @var obj database connection object
	 */
	private $_db_connection = FALSE;
	/**
	 * @var string name of the weblog database
	 */
	private $_database;

	/**
	 * Constructor
	 *
	 * @param string name of weblog database
	 * @param string host of weblog database
	 * @param string username for weblog database
	 * @param string password for weblog database
	 */
	public function __construct($db, $host, $user, $pass)	{
		
		if (! extension_loaded('mysqli')) {
			trigger_error('You need the "mysqli" extension to run this program', E_USER_ERROR);
		}
		
		$this->_database = $db;
		$this->_db_connection = mysqli_connect($host, $user, $pass);
		
		// Ensure the database exists
		if (!mysqli_select_db($this->_db_connection, $this->_database)) {
			$this->_db_connection = FALSE;
		}
	}

	/**
	 * Method to perform queries to SELECT from the database
	 *
	 * @param string SQL query with placeholders if needed
	 * @param array array with values for placeholders
	 * @return array numeric array of found associative rows from the database
	 */
	public function fetch($sql, $params = array())	{
		// Must ensure correct DB is selected. If some other PHP
		// code has run it might have changed the db in between
		// Weblog() and insert().		
		if (!mysqli_select_db($this->_db_connection, $this->_database))
			return;

		$array = array();
		
		if (empty($params)) {
			
			$result = mysqli_query($this->_db_connection, $sql);
			while ($row = mysqli_fetch_assoc($result)) {
			   $array[] = $row;
			}
			mysqli_free_result($result);

		} else {
		
			$stmt = $this->_mysqli_stmt_prepare_execute($sql, $params);
			$result = mysqli_stmt_result_metadata($stmt);
			$fieldinfo = mysqli_fetch_fields($result);
				
			$vars = array();
			$row = array();
			foreach($fieldinfo as $val) {
				// See http://www.php.net/manual/en/function.mysqli-stmt-bind-result.php
				// generate variables for storing result
				$vars[] =& $row[$val->name]; 
			}

			// reference to statement must be first parameter of call to mysqli_stmt_bind_result
			array_unshift($vars, $stmt);
			call_user_func_array('mysqli_stmt_bind_result', $vars);
			$i = 0;
			while(mysqli_stmt_fetch($stmt)) {
				// $array[] = $row seems to assign a reference instead of a copy
				foreach($row as $k => $v) {
					$array[$i][$k] = $v;
				}
				$i++;
			}
			
			mysqli_stmt_close($stmt);
		}

		return $array;
	}

	/**
	 * Method to perform queries that change the database (INSERT, UPDATE, DELETE)
	 *
	 * @param string SQL query with placeholders
	 * @param array array with values for placeholders
	 * @return int number of affected rows
	 */
	public function change($sql, $params) {

		if (!mysqli_select_db($this->_db_connection, $this->_database))
			return;
		
		$stmt = $this->_mysqli_stmt_prepare_execute($sql, $params);
		$numrows = mysqli_stmt_affected_rows($stmt);
		mysqli_stmt_close($stmt);
		
		return $numrows;
	}

	/**
	 * Private method to prepare and execute a mysqli prepared statement
	 *
	 * @param string SQL query with placeholders
	 * @param array array with values for placeholders
	 * @return obj mysqli_stmt object
	 */
	private function _mysqli_stmt_prepare_execute($sql, $params) {
	
		$stmt = mysqli_prepare($this->_db_connection, $sql);
		$types = '';
		$vars = array();
		foreach($params as $key => $param) {
			$vars[] = $params[$key];
			if (is_integer($param)) {
				$types .= 'i';
			} elseif (is_string($param)) {
				$types .= 's';
			}
		}
		
		// string of types must be second parameter of call to mysqli_stmt_bind_param
		array_unshift($vars, $types);
		// reference to statement must be first parameter of call to mysqli_stmt_bind_param
		array_unshift($vars, $stmt);
		call_user_func_array('mysqli_stmt_bind_param', $vars);
		mysqli_stmt_execute($stmt);

		return $stmt;
	}

	/**
	 * Method to test if connection to database is established
	 *
	 * @return bool FALSE if not connected, TRUE otherwise
	 */
	public function connected() {
		
		if ($this->_db_connection === FALSE) {
			return FALSE;
		} else {
			return TRUE;
		}	
	}
}

?>